#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Core things for Melissa platform.
"""
import logging

MELISSA_URL = 'http://developer-api.seemelissa.com/v1/%s'
CLIENT_DATA = {
    'client_id': 'mclimate',
    'client_secret': 'mclimate_core'
}
HEADERS = {
    'User-Agent': 'Mozilla/5.0 (Linux; Android 7.0; SM-G935F Build/NRD90M; wv)'
                  ' AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0'
                  ' Chrome/63.0.3239.111 Mobile Safari/537.36',
    'Connection': 'keep-alive',
    'Pragma': 'no-cache',
    'Cache-Control': 'no-cache',
    'Accept': 'application/json, text/plain, */*',
    'Accept-Encoding': 'gzip, deflate',
    'Accept-Language': 'en-US,sv-SE;q=0.9',
    'X-Requested-With': 'at.cloudfaces.melissa'
}

CHANGE_THRESHOLD = 10
MIN_HUMIDITY_ALLOWED = 10
CHANGE_TIME_CACHE_DEFAULT = 120  # 2 min default

_LOGGER = logging.getLogger(__name__)


class CoreMelissa:
    """
    Core class for Melissa
    """
    SERIAL_NUMBER = 'serial_number'
    COMMAND = 'command'
    STATE = 'state'
    MODE = 'mode'
    TEMP = 'temp'
    FAN = 'fan'
    HUMIDITY = 'humidity'

    FAN_AUTO = 0
    FAN_LOW = 1
    FAN_MEDIUM = 2
    FAN_HIGH = 3

    STATE_OFF = 0
    STATE_ON = 1
    STATE_IDLE = 2

    MODE_AUTO = 0
    MODE_FAN = 1
    MODE_HEAT = 2
    MODE_COOL = 3
    MODE_DRY = 4

    DEFAULT_DATA_MELISSA = {
        SERIAL_NUMBER: "",
        COMMAND: "send_ir_code",
        STATE: STATE_ON,  # 0-OFF; 1-ON; 2-Idle
        MODE: MODE_HEAT,  # 0-Auto; 1-Fan; 2-Heat; 3-Cool; 4-Dry
        TEMP: 20,  # number between 16 and 30 (depends on the codeset)
        FAN: FAN_MEDIUM  # 0-Auto; 1-Low; 2-Med; 3-High
    }

    DEFAULT_DATA_BOBBIE = {
        SERIAL_NUMBER: "",
        COMMAND: "switch_on_off",
        STATE: 'on'
    }

    def __init__(self, **kwargs):
        self.default_headers = kwargs.get('headers', HEADERS)
        self.access_token = kwargs.get('access_token', None)
        self.token_type = kwargs.get('token_type', None)
        self._latest_status = {}

    def _get_headers(self):
        headers = self.default_headers.copy()
        headers.update(
            {'Authorization': "%s %s" % (self.token_type, self.access_token)}
        )
        _LOGGER.debug(headers)
        return headers

    def sanity_check(self, data, device):
        ret = True
        if self._latest_status and self._latest_status.get(device) and abs(
                data[self.TEMP] - self._latest_status[device][self.TEMP]
        ) > CHANGE_THRESHOLD:
            ret = False
        if self._latest_status and self._latest_status.get(device) and \
                data[self.HUMIDITY] < MIN_HUMIDITY_ALLOWED:
            ret = False
        return ret

    @property
    def have_connection(self):
        return self.access_token is not None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Async version of the wrapper.
"""
import json
import logging
from datetime import datetime

import aiohttp
import requests
from melissa.exceptions import ApiException, UnsupportedDevice

from melissa import CHANGE_TIME_CACHE_DEFAULT, HEADERS, MELISSA_URL, \
    CLIENT_DATA
from melissa.core import CoreMelissa

__author__ = 'Magnus Knutas'
LOGGER = logging.getLogger(__name__)


class AsyncMelissa(CoreMelissa):
    """
    Async class for Melissa.
    """
    session = None

    def __init__(self, **kwargs):
        super(AsyncMelissa, self).__init__(**kwargs)
        self.session = aiohttp.ClientSession(connector_owner=False)
        self.username = kwargs['username']
        self.password = kwargs['password']
        self.refresh_token = kwargs.get('refresh_token', None)
        self.devices = {}
        self.geofences = {}
        self._latest_humidity = None
        self._latest_temp = None
        self._latest_status = {}
        self._time_cache = kwargs.get('time_cache', CHANGE_TIME_CACHE_DEFAULT)
        self.fetch_timestamp = None

    async def async_connect(self):
        url = MELISSA_URL % 'auth/login'
        LOGGER.info(url)
        data = CLIENT_DATA.copy()
        data.update(
            {'username': self.username, 'password': self.password}
        )
        LOGGER.info(data)
        req = await self.session.post(
            url, data=data,
            headers=HEADERS
        )
        if req.status == requests.codes.ok:
            resp = json.loads(await req.text())
            self.access_token = resp['auth']['access_token']
            self.refresh_token = resp['auth']['refresh_token']
            self.token_type = resp['auth']['token_type']
        else:
            raise ApiException(await req.text(), req.status)

    async def async_fetch(self, url):
        response = await self.session.get(url)
        ret = await response.text()
        return ret

    async def async_fetch_devices(self):
        url = MELISSA_URL % 'controllers'
        LOGGER.info(url)
        headers = self._get_headers()
        req = await self.session.get(url, headers=headers)
        if req.status == requests.codes.ok:
            resp = json.loads(await req.text())
            for controller in resp['_embedded']['controller']:
                self.devices[controller['serial_number']] = controller
        LOGGER.debug(self.devices)
        return self.devices

    async def async_fetch_geofences(self):
        url = MELISSA_URL % 'geofences'
        LOGGER.info(url)
        headers = self._get_headers()
        req = await self.session.get(url, headers=headers)
        if req.status == requests.codes.ok:
            resp = json.loads(await req.text())
            for geofence in resp['_embedded']['geofence']:
                self.geofences[geofence['controller_id']] = geofence
        LOGGER.info(self.geofences)
        return self.geofences

    async def async_send(self, device, device_type='melissa', state_data=None):
        if device_type == 'melissa':
            data = self.DEFAULT_DATA_MELISSA.copy()
        elif device_type == 'bobbie':
            data = self.DEFAULT_DATA_BOBBIE.copy()
        else:
            raise UnsupportedDevice(device_type)

        if state_data:
            data.update(state_data)

        data.update({self.SERIAL_NUMBER: device})
        url = MELISSA_URL % 'provider/send'
        LOGGER.info(url)

        if not self.have_connection:
            await self.async_connect()

        headers = self._get_headers()
        headers.update({'Content-Type': 'application/json'})

        input_data = json.dumps(data)
        LOGGER.debug(input_data)
        req = await self.session.post(url, data=input_data, headers=headers)
        if not req.status == requests.codes.ok:
            raise ApiException("%s - %s", (req.status, req.text()))

        return True

    async def async_status(self, test=False, cached=False):
        # TODO: Update self._send_cache
        if cached and self.fetch_timestamp and self._time_cache > \
                (datetime.utcnow() - self.fetch_timestamp).total_seconds():
            return self._latest_status
        url = MELISSA_URL % 'provider/fetch'
        LOGGER.info(url)
        headers = self._get_headers()
        headers.update({'Content-Type': 'application/json'})
        ret = {}
        if not self.devices:
            await self.async_fetch_devices()
        for device in self.devices:
            if self.devices[device]['type'] in ('melissa', 'bobbie'):
                input_data = json.dumps({'serial_number': device})
                req = await self.session.post(
                    url, data=input_data, headers=headers)
                if req.status == requests.codes.ok:
                    data = json.loads(await req.text())
                    if self.devices[device]['type'] == 'bobbie':
                        ret[device] = data['provider']
                    elif self.sanity_check(data['provider'], device):
                        ret[device] = data['provider']
                    else:
                        ret[device] = self._latest_status[device]
                elif req.status == requests.codes.unauthorized and \
                        not test:
                    await self.async_connect()
                    return self.async_status()
                else:
                    raise ApiException(await req.text())
        self.fetch_timestamp = datetime.utcnow()
        self._latest_status = ret
        return ret

    async def async_cur_settings(self, serial_number):
        url = MELISSA_URL % 'controllers/%s' % serial_number
        LOGGER.info(url)
        headers = self._get_headers()
        req = await self.session.get(
                url, headers=headers)
        if req.status == requests.codes.ok:
            data = json.loads(await req.text())
        else:
            raise ApiException(await req.text())
        LOGGER.debug(data)
        return data
